--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       data_path.vhd
--
--  Description :     This module comprises the write and read data paths for the
--                    DDR1 memory interface. The write data along with write enable 
--                    signals are forwarded to the DDR IOB FFs. The read data is 
--                    captured in CLB FFs and finally input to FIFOs.
-- 
--                    
--  Date - revision : 10/16/2003
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use work.xst_vhdl_bl4cl3_parameters_0.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity xst_vhdl_bl4cl3_data_path_0 is
port(
     user_input_data    : in std_logic_vector(((data_width*2)-1) downto 0);
     user_data_mask     : in std_logic_vector(((data_mask_width*2) -1) downto 0);
     clk                : in std_logic;
     clk90              : in std_logic;
    -- clk180             : in std_logic; -- signal added to the entity
    -- clk270             : in std_logic; -- signal added to the entity
     reset              : in std_logic;
     reset90            : in std_logic;
     reset180           : in std_logic;
     reset270           : in std_logic;
     write_enable       : in std_logic;
     rst_dqs_div_in     : in std_logic;
     delay_sel          : in std_logic_vector(4 downto 0); 
     
     
	dqs_int_delay_in0 : in std_logic; 

	dqs_int_delay_in1 : in std_logic;  
     dq                 : in std_logic_vector((data_width-1) downto 0);       
     u_data_val         : out std_logic;
     user_output_data   : out std_logic_vector(((data_width*2)-1) downto 0);
     write_en_val       : out std_logic;
     reset90_r_val      : out std_logic;

     data_mask_f       : out std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : out std_logic_vector((data_mask_width-1) downto 0);


     write_data_falling : out std_logic_vector((data_width-1) downto 0);
     write_data_rising  : out std_logic_vector((data_width-1) downto 0)
     );
end xst_vhdl_bl4cl3_data_path_0 ;

architecture arc of xst_vhdl_bl4cl3_data_path_0  is

component xst_vhdl_bl4cl3_data_read_0 
port(

     clk90              : in std_logic;
     reset90_r          : in std_logic;

     ddr_dq_in          : in std_logic_vector((data_width-1) downto 0);   
     read_valid_data_1  : in std_logic;
  
     
        fifo_00_wr_en      : in std_logic;

        fifo_10_wr_en      : in std_logic;
      
        
     
        fifo_01_wr_en      : in std_logic;

        fifo_11_wr_en      : in std_logic;
    
     
        fifo_00_wr_addr    : in std_logic_vector(3 downto 0);
        fifo_01_wr_addr    : in std_logic_vector(3 downto 0); 

        fifo_10_wr_addr    : in std_logic_vector(3 downto 0);
        fifo_11_wr_addr    : in std_logic_vector(3 downto 0); 
 


		
        dqs0_delayed_col1      : in std_logic;

        dqs1_delayed_col1      : in std_logic;
		
            
        dqs0_delayed_col0      : in std_logic;

        dqs1_delayed_col0      : in std_logic;
		


		user_output_data   : out std_logic_vector(((data_width*2)-1) downto 0);
		fifo0_rd_addr_val: out std_logic_vector(3 downto 0);
               fifo1_rd_addr_val: out std_logic_vector(3 downto 0)     

     );
end component;


component   xst_vhdl_bl4cl3_data_read_controller_0
port(

     clk90              : in std_logic;
     reset_r            : in std_logic;
     reset90_r          : in std_logic;
     rst_dqs_div_in     : in std_logic;
     delay_sel          : in std_logic_vector(4 downto 0);   

     
        dqs_int_delay_in0      : in std_logic;

        dqs_int_delay_in1      : in std_logic;
  
    
     fifo0_rd_addr      : in std_logic_vector(3 downto 0);
     fifo1_rd_addr      : in std_logic_vector(3 downto 0);
     u_data_val   	: out std_logic;   
  
     
     
        fifo_00_wr_en_val      : out std_logic;

        fifo_10_wr_en_val      : out std_logic;

     
        fifo_01_wr_en_val      : out std_logic;

        fifo_11_wr_en_val      : out std_logic; 
   
     
        fifo_00_wr_addr_val      : out std_logic_vector(3 downto 0);
        fifo_01_wr_addr_val      : out std_logic_vector(3 downto 0);

        fifo_10_wr_addr_val      : out std_logic_vector(3 downto 0);
        fifo_11_wr_addr_val      : out std_logic_vector(3 downto 0);



          
        dqs0_delayed_col0_val      : out std_logic;

        dqs1_delayed_col0_val      : out std_logic;

	  
        dqs0_delayed_col1_val      : out std_logic;

        dqs1_delayed_col1_val      : out std_logic;
		
	 read_valid_data_1_val  : out std_logic	

      );

end component;



component xst_vhdl_bl4cl3_data_write_0
port(
     user_input_data    : in std_logic_vector(((data_width*2)-1) downto 0);
     user_data_mask     : in std_logic_vector(((data_mask_width*2) -1) downto 0);
     clk90              : in std_logic;
 --    clk270             : in std_logic; -- signal added
     reset90_r          : in std_logic;
     reset270_r         : in std_logic;
     write_enable       : in std_logic;
     write_data_falling : out std_logic_vector((data_width-1) downto 0);
     write_data_rising  : out std_logic_vector((data_width-1) downto 0);

     data_mask_f       : out std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : out std_logic_vector((data_mask_width-1) downto 0);


     write_en_val       : out std_logic
     );
end component;   

component  xst_vhdl_bl4cl3_data_path_rst
port(
     clk                : in std_logic;
     clk90              : in std_logic;
  --   clk180             : in std_logic;       -- clk180 added 
  --   clk270             : in std_logic;       -- clk270 added 
     reset              : in std_logic;
     reset90            : in std_logic;
     reset180           : in std_logic;
     reset270           : in std_logic;
     reset_r            : out std_logic;
     reset90_r          : out std_logic;
     reset180_r         : out std_logic;
     reset270_r         : out std_logic
    );
end component;

signal reset_r          : std_logic;
signal reset90_r        : std_logic;
signal reset180_r       : std_logic;
signal reset270_r       : std_logic;
                                          
                                 
 signal fifo0_rd_addr    : std_logic_vector(3 downto 0);
 signal fifo1_rd_addr    : std_logic_vector(3 downto 0);                          
 signal read_valid_data_1  : std_logic;
 
 
        signal fifo_00_wr_addr      :  std_logic_vector(3 downto 0);
        signal fifo_01_wr_addr      :  std_logic_vector(3 downto 0); 

        signal fifo_10_wr_addr      :  std_logic_vector(3 downto 0);
        signal fifo_11_wr_addr      :  std_logic_vector(3 downto 0); 
      

 
        signal fifo_00_wr_en        :  std_logic; 

        signal fifo_10_wr_en        :  std_logic; 

 
 
        signal fifo_01_wr_en        :  std_logic; 

        signal fifo_11_wr_en        :  std_logic; 
 

 
        signal dqs0_delayed_col1        :  std_logic; 

        signal dqs1_delayed_col1        :  std_logic; 


 
        signal dqs0_delayed_col0        :  std_logic; 

        signal dqs1_delayed_col0        :  std_logic; 
 

 begin

reset90_r_val <= reset90_r;

data_read0 : xst_vhdl_bl4cl3_data_read_0
port map (
           clk90               =>    clk90,
           reset90_r           =>    reset90_r,
	     ddr_dq_in           =>    dq,
           read_valid_data_1   =>    read_valid_data_1,
   
	    
        fifo_00_wr_en	=> fifo_00_wr_en, 

        fifo_10_wr_en	=> fifo_10_wr_en, 
	   	   
            
        fifo_01_wr_en	=> fifo_01_wr_en, 

        fifo_11_wr_en	=> fifo_11_wr_en, 
	  
            
        fifo_00_wr_addr	=> fifo_00_wr_addr, 
        fifo_01_wr_addr	=> fifo_01_wr_addr,

        fifo_10_wr_addr	=> fifo_10_wr_addr, 
        fifo_11_wr_addr	=> fifo_11_wr_addr,
         
	    
        dqs0_delayed_col0  => dqs0_delayed_col0, 

        dqs1_delayed_col0  => dqs1_delayed_col0, 	     
   		      
          
        dqs0_delayed_col1  => dqs0_delayed_col1, 

        dqs1_delayed_col1  => dqs1_delayed_col1, 
   		      		     
           user_output_data    =>    user_output_data,
           fifo0_rd_addr_val   =>    fifo0_rd_addr,
           fifo1_rd_addr_val   =>    fifo1_rd_addr
         );



data_read_controller0 : xst_vhdl_bl4cl3_data_read_controller_0
port map (
            clk90              =>   clk90,
            reset_r            =>   reset_r,
            reset90_r          =>   reset90_r,
            rst_dqs_div_in     =>   rst_dqs_div_in,
            delay_sel          =>   delay_sel,
	     
        dqs_int_delay_in0  => dqs_int_delay_in0, 

        dqs_int_delay_in1  => dqs_int_delay_in1, 
       
            fifo0_rd_addr    =>   fifo0_rd_addr,
            fifo1_rd_addr    =>   fifo1_rd_addr,
            u_data_val       => u_data_val,
         
            
        fifo_00_wr_en_val  => fifo_00_wr_en, 

        fifo_10_wr_en_val  => fifo_10_wr_en, 
                   
            
        fifo_01_wr_en_val  => fifo_01_wr_en, 

        fifo_11_wr_en_val  => fifo_11_wr_en, 
	              
	      
        fifo_00_wr_addr_val  => fifo_00_wr_addr, 
        fifo_01_wr_addr_val  => fifo_01_wr_addr,

        fifo_10_wr_addr_val  => fifo_10_wr_addr, 
        fifo_11_wr_addr_val  => fifo_11_wr_addr,
                    
	      
        dqs0_delayed_col0_val  => dqs0_delayed_col0, 

        dqs1_delayed_col0_val  => dqs1_delayed_col0, 
            
            
        dqs0_delayed_col1_val  => dqs0_delayed_col1, 

        dqs1_delayed_col1_val  => dqs1_delayed_col1, 

	      read_valid_data_1_val  =>   read_valid_data_1
				
         );

         
data_write0 :  xst_vhdl_bl4cl3_data_write_0
port map (
          user_input_data    =>   user_input_data,
	    user_data_mask     =>   user_data_mask,
          clk90              =>   clk90,
          --clk270             =>   clk270,
          reset90_r          =>   reset90_r,
          reset270_r         =>   reset270_r,
          write_enable       =>   write_enable,
          write_en_val       =>   write_en_val,
          write_data_falling =>   write_data_falling,
          write_data_rising  =>   write_data_rising,
          data_mask_f        =>   data_mask_f,
          data_mask_r        =>   data_mask_r
         );

data_path_rst0 :  xst_vhdl_bl4cl3_data_path_rst
port map (
          clk                =>   clk,            
          clk90              =>   clk90,
	  --  clk180             =>   clk180,
        --  clk270             =>   clk270,
          reset              =>   reset,
          reset90            =>   reset90,
          reset180           =>   reset180,
          reset270           =>   reset270,
          reset_r            =>   reset_r,
          reset90_r          =>   reset90_r,
          reset180_r         =>   reset180_r,
          reset270_r         =>   reset270_r
         );
                                                                       
         
end arc ;


