--*********************************************************************
-- DDR 72 Bit Controller DATA PATH for LEFT RIGHT Pins

-- In the current DATA PATH logic DATA CAPTURE part was modified.
-- The below changes were made to reduce the resources in 
-- the data capture

-- in the current architecture data ( dq ) from ddr memory 
-- directly stored into the FIFO's.

-- Architectural changes :

-- Used only TWO FIFOs ( instead of FOUR FIFOs ) 
-- Used Single col ( col0 ) dqs_delayed_col signals
-- Used Gray Counters for write and read pointers of the FIFOs 

-- fbit stage is removed from ddr1_dqbit module ( in the data capture )
-- dq_clk stage was removed 
-- dqs_clk_div logic was removed
-- ddr1_transfer_done logic was removed 
-- data valid signals registering in clk90 domain was removed

-- fifo_0 and fifo_1 wr_addr was double registered in clk90 domain
-- only fifo_0 and fifo_1 empty signals were used for read_data_valid_1 logic  
-- write enable for the FIFOs derived from rst_dqs_div signal


-- Code revised by 	: Narayana Murty.
-- Date 			: Nov 18, 2003. 

--*********************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
use work.xst_vhdl_bl4cl3_parameters_0.all;

entity    xst_vhdl_bl4cl3_data_path_iobs_0  is
port(
    clk               : in std_logic;
    clk90             : in std_logic;
  --  clk180            : in STD_LOGIC; 
  --  clk270            : in STD_LOGIC; 
    reset90_r         : in std_logic;
    dqs_reset         : in std_logic;
    dqs_enable        : in std_logic;

ddr_dqs               : inout std_logic_vector((data_strobe_width -1) downto 0);


    ddr_dq            : inout std_logic_vector((data_width-1) downto 0);
    write_data_falling: in std_logic_vector((data_width-1) downto 0);
    write_data_rising : in std_logic_vector((data_width-1) downto 0);
    write_en_val      : in std_logic;

     data_mask_f       : in std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : in std_logic_vector((data_mask_width-1) downto 0);


    
     dqs_int_delay_in0  : out std_logic ; 

     dqs_int_delay_in1  : out std_logic ; 


ddr_dm                : out std_logic_vector((data_mask_width-1) downto 0);

  
    ddr_dq_val        : out std_logic_vector((data_width-1) downto 0)

);
end   xst_vhdl_bl4cl3_data_path_iobs_0 ;  


architecture arc of xst_vhdl_bl4cl3_data_path_iobs_0 is


attribute syn_keep : boolean;  -- Using Syn_Keep Directive


component  xst_vhdl_bl4cl3_s3_dqs_iob
port(
     clk            : in std_logic;
  -- clk180         : in std_logic;
     ddr_dqs_reset  : in std_logic;
     ddr_dqs_enable : in std_logic;
     ddr_dqs        : inout std_logic;
     dqs            : out std_logic
     );
end component;

component  xst_vhdl_bl4cl3_s3_ddr_iob
port (
      ddr_dq_inout       : inout std_logic; --Bi-directional SDRAM data bus
      write_data_falling : in std_logic;    --Transmit data, output on falling edge
      write_data_rising  : in std_logic;    --Transmit data, output on rising edge
      read_data_in       : out std_logic;   -- Received data
      clk90              : in std_logic;    --Clock 90
    --clk270             : in std_logic;
      write_en_val       : in std_logic;    --Transmit enable
      reset              : in std_logic); 

end component;

component	xst_vhdl_bl4cl3_ddr1_dm_0
port (                                                

ddr_dm                : out std_logic_vector((data_mask_width-1) downto 0);

 

      mask_falling : in std_logic_vector((data_mask_width-1) downto 0);    --Mask output on falling edge



      mask_rising : in std_logic_vector((data_mask_width-1) downto 0);    --Mask output on falling edge


      clk90        : in std_logic    
--    clk270       : in std_logic                     
      );                                              
end component;                                        

  signal ddr_dq_in    : std_logic_vector((data_width-1) downto 0);

begin

ddr_dq_val <= ddr_dq_in;

--ddr1_dm0	:	xst_vhdl_bl4cl3_ddr1_dm_0	port	map	( 
--		       		   ddr_dm       => ddr_dm,
--         				   mask_falling => data_mask_f,
--                                 mask_rising  => data_mask_r,
--					   clk90        => clk90,
--                                 clk270       => clk270
--                            );


ddr1_dm0    :       xst_vhdl_bl4cl3_ddr1_dm_0	port	map	( 
		       		   ddr_dm       => ddr_dm,
       				   mask_falling => data_mask_f,
                                   mask_rising  => data_mask_r,
				   clk90        => clk90
                                 --  clk270       => clk270
                            );



--***********************************************************************
--    Read Data Capture Module Instantiations
--***********************************************************************
-- DQS IOB instantiations
--***********************************************************************


 
 s3_dqs_iob0 : xst_vhdl_bl4cl3_s3_dqs_iob port map (
                                clk             => clk,
                             -- clk180          => clk180,
                                ddr_dqs_reset   => dqs_reset,
                                ddr_dqs_enable  => dqs_enable,
                                ddr_dqs         => ddr_dqs(0),
                                dqs             => dqs_int_delay_in0      
                                          );


 s3_dqs_iob1 : xst_vhdl_bl4cl3_s3_dqs_iob port map (
                                clk             => clk,
                             -- clk180          => clk180,
                                ddr_dqs_reset   => dqs_reset,
                                ddr_dqs_enable  => dqs_enable,
                                ddr_dqs         => ddr_dqs(1),
                                dqs             => dqs_int_delay_in1      
                                          );

 
                             

--******************************************************************************************************************************
-- DDR Data bit instantiations 
--******************************************************************************************************************************            


 s3_ddr_iob0 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(0), 
      				 write_data_falling => write_data_falling(0), 
      				 write_data_rising  => write_data_rising(0),
      				 read_data_in       => ddr_dq_in(0),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob1 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(1), 
      				 write_data_falling => write_data_falling(1), 
      				 write_data_rising  => write_data_rising(1),
      				 read_data_in       => ddr_dq_in(1),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob2 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(2), 
      				 write_data_falling => write_data_falling(2), 
      				 write_data_rising  => write_data_rising(2),
      				 read_data_in       => ddr_dq_in(2),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob3 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(3), 
      				 write_data_falling => write_data_falling(3), 
      				 write_data_rising  => write_data_rising(3),
      				 read_data_in       => ddr_dq_in(3),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob4 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(4), 
      				 write_data_falling => write_data_falling(4), 
      				 write_data_rising  => write_data_rising(4),
      				 read_data_in       => ddr_dq_in(4),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob5 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(5), 
      				 write_data_falling => write_data_falling(5), 
      				 write_data_rising  => write_data_rising(5),
      				 read_data_in       => ddr_dq_in(5),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob6 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(6), 
      				 write_data_falling => write_data_falling(6), 
      				 write_data_rising  => write_data_rising(6),
      				 read_data_in       => ddr_dq_in(6),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob7 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(7), 
      				 write_data_falling => write_data_falling(7), 
      				 write_data_rising  => write_data_rising(7),
      				 read_data_in       => ddr_dq_in(7),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob8 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(8), 
      				 write_data_falling => write_data_falling(8), 
      				 write_data_rising  => write_data_rising(8),
      				 read_data_in       => ddr_dq_in(8),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob9 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(9), 
      				 write_data_falling => write_data_falling(9), 
      				 write_data_rising  => write_data_rising(9),
      				 read_data_in       => ddr_dq_in(9),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob10 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(10), 
      				 write_data_falling => write_data_falling(10), 
      				 write_data_rising  => write_data_rising(10),
      				 read_data_in       => ddr_dq_in(10),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob11 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(11), 
      				 write_data_falling => write_data_falling(11), 
      				 write_data_rising  => write_data_rising(11),
      				 read_data_in       => ddr_dq_in(11),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob12 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(12), 
      				 write_data_falling => write_data_falling(12), 
      				 write_data_rising  => write_data_rising(12),
      				 read_data_in       => ddr_dq_in(12),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob13 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(13), 
      				 write_data_falling => write_data_falling(13), 
      				 write_data_rising  => write_data_rising(13),
      				 read_data_in       => ddr_dq_in(13),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob14 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(14), 
      				 write_data_falling => write_data_falling(14), 
      				 write_data_rising  => write_data_rising(14),
      				 read_data_in       => ddr_dq_in(14),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             


 s3_ddr_iob15 : xst_vhdl_bl4cl3_s3_ddr_iob port map (
                                 ddr_dq_inout       => ddr_dq(15), 
      				 write_data_falling => write_data_falling(15), 
      				 write_data_rising  => write_data_rising(15),
      				 read_data_in       => ddr_dq_in(15),
      				 clk90              => clk90,
      			--	 clk270             => clk270,
      				 write_en_val       => write_en_val,
      				 reset              => reset90_r
                                                    );             
 

end arc;
