--*********************************************************************
-- DDR 64 Bit Controller DATA PATH for LEFT RIGHT Pins

-- In the current DATA PATH logic DATA CAPTURE part was modified.
-- The below changes were made to reduce the resources in 
-- the data capture

-- in the current architecture data ( dq ) from ddr memory 
-- directly stored into the FIFO's.

-- Architectural changes :

-- Used only TWO FIFOs ( instead of FOUR FIFOs ) 
-- Used Single col ( col0 ) dqs_delayed_col signals
-- Used Gray Counters for write and read pointers of the FIFOs 

-- fbit stage is removed from ddr1_dqbit module ( in the data capture )
-- dq_clk stage was removed 
-- dqs_clk_div logic was removed
-- ddr1_transfer_done logic was removed 
-- data valid signals registering in clk90 domain was removed

-- fifo_0 and fifo_1 wr_addr was double registered in clk90 domain
-- only fifo_0 and fifo_1 empty signals were used for read_data_valid_1 logic  
-- write enable for the FIFOs derived from rst_dqs_div signal


-- Code revised by 	: Narayana Murty.
-- Date 			      : Nov 18, 2003. 

--*********************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.xst_vhdl_bl4cl3_parameters_0.all;
--use work.RAM8D_0.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity xst_vhdl_bl4cl3_data_read_0 is
port(

     clk90              : in std_logic;
     reset90_r          : in std_logic;
     ddr_dq_in          : in std_logic_vector((data_width-1) downto 0);   
     read_valid_data_1  : in std_logic;
     
     
                 fifo_00_wr_en    : in std_logic;
                 fifo_01_wr_en    : in std_logic;

                 fifo_10_wr_en    : in std_logic;
                 fifo_11_wr_en    : in std_logic;
 
     
              fifo_00_wr_addr : in std_logic_vector(3 downto 0);
              fifo_01_wr_addr : in std_logic_vector(3 downto 0);

              fifo_10_wr_addr : in std_logic_vector(3 downto 0);
              fifo_11_wr_addr : in std_logic_vector(3 downto 0);
     
 
     
                 dqs0_delayed_col0  : in std_logic;
                 dqs0_delayed_col1  : in std_logic;

                 dqs1_delayed_col0  : in std_logic;
                 dqs1_delayed_col1  : in std_logic;
     
     
     user_output_data   : out std_logic_vector(((data_width*2)-1) downto 0);
     fifo0_rd_addr_val: out std_logic_vector(3 downto 0);
     fifo1_rd_addr_val: out std_logic_vector(3 downto 0)     

     );
end xst_vhdl_bl4cl3_data_read_0 ;


architecture arc of xst_vhdl_bl4cl3_data_read_0  is

--attribute syn_noprune  : boolean;  
--attribute syn_preserve : boolean; 


-- rd_gray_cntr is a gray counter with a SYNC reset ( reset_90r) for fifo rd_addr
component xst_vhdl_bl4cl3_rd_gray_cntr port (
			clk 				: 	in std_logic;         
			reset				:	in std_logic;
			cnt_en                  :	in std_logic; 
                  rgc_gcnt                :     out std_logic_vector(3 downto 0)
                        
		  );
end component; 

component FD port(
      Q        : out STD_LOGIC;
      C        : in STD_LOGIC;
      D        : in STD_LOGIC
      );
end component;


component  xst_vhdl_bl4cl3_RAM8D_0  is
port (
        DOUT  : out std_logic_vector((DatabitsPerReadClock -1) downto 0);
        WADDR : in std_logic_vector(3 downto 0);
	  DIN   : in std_logic_vector((DatabitsPerReadClock -1) downto 0);
	  RADDR : in std_logic_vector(3 downto 0);
	  WCLK0 : in std_logic;
	  WCLK1 : in std_logic;
	  WE    : in std_logic
	 );
end component;

signal read_valid_data_1_r   : std_logic;
signal read_valid_data_1_r1   : std_logic;
signal read_valid_data_1_r2   : std_logic;



signal fifo00_rd_addr        : std_logic_vector(3 downto 0);
signal fifo01_rd_addr        : std_logic_vector(3 downto 0);


              signal fifo00_rd_addr_r  : std_logic_vector(3 downto 0);
              signal fifo01_rd_addr_r  : std_logic_vector(3 downto 0);

              signal fifo10_rd_addr_r  : std_logic_vector(3 downto 0);
              signal fifo11_rd_addr_r  : std_logic_vector(3 downto 0);


signal fifop_rd_addr_r         : std_logic_vector(3 downto 0);
--attribute syn_noprune   of fifop_rd_addr_r  : signal is true;
--attribute syn_preserve  of fifop_rd_addr_r  : signal is true;









             signal fifo_00_data_out  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);
             signal fifo_01_data_out  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);

             signal fifo_10_data_out  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);
             signal fifo_11_data_out  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);


-- reg added for timing 


             signal fifo_00_data_out_r  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);
             signal fifo_01_data_out_r  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);

             signal fifo_10_data_out_r  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);
             signal fifo_11_data_out_r  : std_logic_vector(DatabitsPerReadClock - 1 downto 0);


signal first_sdr_data         : std_logic_vector(((data_width*2)-1) downto 0);


             signal dqs0_delayed_col0_n  : std_logic; 

             signal dqs1_delayed_col0_n  : std_logic; 



             signal dqs0_delayed_col1_n  : std_logic; 

             signal dqs1_delayed_col1_n  : std_logic; 


begin


              dqs0_delayed_col0_n  <= not dqs0_delayed_col0 ; 
              dqs0_delayed_col1_n  <= not dqs0_delayed_col1 ;

              dqs1_delayed_col0_n  <= not dqs1_delayed_col0 ; 
              dqs1_delayed_col1_n  <= not dqs1_delayed_col1 ;


user_output_data    <= first_sdr_data;


fifo0_rd_addr_val   <= fifo01_rd_addr;
fifo1_rd_addr_val   <= fifo00_rd_addr;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then

    
              fifo_00_data_out_r <= (others => '0'); 
              fifo_01_data_out_r <= (others => '0'); 

              fifo_10_data_out_r <= (others => '0'); 
              fifo_11_data_out_r <= (others => '0'); 

   
 else
    
    
              fifo_00_data_out_r  <= fifo_00_data_out;
              fifo_01_data_out_r  <= fifo_01_data_out; 

              fifo_10_data_out_r  <= fifo_10_data_out;
              fifo_11_data_out_r  <= fifo_11_data_out; 
 
end if;                                                    
end if;                                                    
end process;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then

    
              fifo00_rd_addr_r  <= "0000";
              fifo01_rd_addr_r  <= "0000"; 

              fifo10_rd_addr_r  <= "0000";
              fifo11_rd_addr_r  <= "0000"; 

    fifop_rd_addr_r  <= "0000";
else
    
              fifo00_rd_addr_r  <= fifo00_rd_addr;
              fifo01_rd_addr_r  <= fifo01_rd_addr; 

              fifo10_rd_addr_r  <= fifo00_rd_addr;
              fifo11_rd_addr_r  <= fifo01_rd_addr; 


    fifop_rd_addr_r  <= fifo01_rd_addr; 
end if;                                                     
end if;                                                    
end process;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then
    first_sdr_data   <= (others => '0');  
    read_valid_data_1_r <= '0';
    read_valid_data_1_r1 <= '0';
    read_valid_data_1_r2 <= '0';
 else
             read_valid_data_1_r <= read_valid_data_1;
             read_valid_data_1_r1 <= read_valid_data_1_r;
             read_valid_data_1_r2 <= read_valid_data_1_r1;
             if (read_valid_data_1_r1 = '1') then    
             	first_sdr_data  <= 
			         ( 
              fifo_10_data_out_r  & 
              fifo_00_data_out_r  & 
              fifo_11_data_out_r  & 
              fifo_01_data_out_r 
				 );
             else
               first_sdr_data  <= first_sdr_data;               
             end if;              
end if;                                                    
end if;                                                    
end process;     
   
--------------------------------------------------------------------

-- fifo_x0_rd_addr and fifo_x1_rd_addr counters ( gray counters )  

fifo0_rd_addr_inst :   xst_vhdl_bl4cl3_rd_gray_cntr port map (
							clk 		=>	clk90,
							reset		=>	reset90_r,
							cnt_en	=>	read_valid_data_1,
                                          rgc_gcnt	=>	fifo00_rd_addr
                                                       
						); 
fifo1_rd_addr_inst :   xst_vhdl_bl4cl3_rd_gray_cntr port map (
							clk 		=>	clk90,
							reset		=>	reset90_r,
							cnt_en	=>	read_valid_data_1,
                                          rgc_gcnt	=>	fifo01_rd_addr
                                                        
						);                                                                         

  
--*************************************************************************************************************************
-- RAM 8D instantiations 
--*************************************************************************************************************************



strobe0 : xst_vhdl_bl4cl3_RAM8D_0
    Port Map (
              DOUT  => fifo_00_data_out((DatabitsPerReadClock-1) downto 0), 
	          WADDR => fifo_00_wr_addr(3 downto 0),
	          DIN   => ddr_dq_in(7 downto 0), 
              RADDR => fifo00_rd_addr_r(3 downto 0), 
	          WCLK0 => dqs0_delayed_col0, 
              WCLK1 => dqs0_delayed_col1,
	          WE    => fifo_00_wr_en
              );

strobe0_n : xst_vhdl_bl4cl3_RAM8D_0
    Port Map (
              DOUT  => fifo_01_data_out((DatabitsPerReadClock-1) downto 0), 
	          WADDR => fifo_01_wr_addr(3 downto 0),
	          DIN   => ddr_dq_in(7 downto 0), 
              RADDR => fifo01_rd_addr_r(3 downto 0), 
	          WCLK0 => dqs0_delayed_col0_n, 
              WCLK1 => dqs0_delayed_col1_n,
	          WE    => fifo_01_wr_en
              );     

strobe1 : xst_vhdl_bl4cl3_RAM8D_0
    Port Map (
              DOUT  => fifo_10_data_out((DatabitsPerReadClock-1) downto 0), 
	          WADDR => fifo_10_wr_addr(3 downto 0),
	          DIN   => ddr_dq_in(15 downto 8), 
              RADDR => fifo10_rd_addr_r(3 downto 0), 
	          WCLK0 => dqs1_delayed_col0, 
              WCLK1 => dqs1_delayed_col1,
	          WE    => fifo_10_wr_en
              );

strobe1_n : xst_vhdl_bl4cl3_RAM8D_0
    Port Map (
              DOUT  => fifo_11_data_out((DatabitsPerReadClock-1) downto 0), 
	          WADDR => fifo_11_wr_addr(3 downto 0),
	          DIN   => ddr_dq_in(15 downto 8), 
              RADDR => fifo11_rd_addr_r(3 downto 0), 
	          WCLK0 => dqs1_delayed_col0_n, 
              WCLK1 => dqs1_delayed_col1_n,
	          WE    => fifo_11_wr_en
              );     

  
end arc;
