--*********************************************************************
-- DDR 72 Bit Controller DATA PATH for LEFT RIGHT Pins

-- In the current DATA PATH logic DATA CAPTURE part was modified.
-- The below changes were made to reduce the resources in 
-- the data capture

-- in the current architecture data ( dq ) from ddr memory 
-- directly stored into the FIFO's.

-- Architectural changes :

-- Used only TWO FIFOs ( instead of FOUR FIFOs ) 
-- Used Single col ( col0 ) dqs_delayed_col signals
-- Used Gray Counters for write and read pointers of the FIFOs 

-- fbit stage is removed from ddr1_dqbit module ( in the data capture )
-- dq_clk stage was removed 
-- dqs_clk_div logic was removed
-- ddr1_transfer_done logic was removed 
-- data valid signals registering in clk90 domain was removed

-- fifo_0 and fifo_1 wr_addr was double registered in clk90 domain
-- only fifo_0 and fifo_1 empty signals were used for read_data_valid_1 logic  
-- write enable for the FIFOs derived from rst_dqs_div signal


-- Code revised by 	: Narayana Murty.
-- Date 			      : Nov 18, 2003. 

--*********************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use work.xst_vhdl_bl4cl3_parameters_0.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--use work.parameter_64bit.all;

entity    xst_vhdl_bl4cl3_data_write_0 is
port(
     user_input_data    : in std_logic_vector(((data_width*2)-1) downto 0);
     user_data_mask     : in std_logic_vector(((data_mask_width*2)-1) downto 0);
     clk90              : in std_logic;
    -- clk270             : in std_logic;-- signal added to entity
     reset90_r          : in std_logic;
     reset270_r         : in std_logic;
     write_enable       : in std_logic;
     write_data_falling : out std_logic_vector(((data_width)-1) downto 0);
     write_data_rising  : out std_logic_vector(((data_width)-1) downto 0);

     data_mask_f       : out std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : out std_logic_vector((data_mask_width-1) downto 0);


     write_en_val       : out std_logic

);
end   xst_vhdl_bl4cl3_data_write_0;  

architecture   arc of   xst_vhdl_bl4cl3_data_write_0 is
component FD
   port(
      Q                              :  out   STD_LOGIC;
      D                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC);
end component;

signal write_en_P1            : std_logic;
signal write_en_P2            : std_logic;
signal write_en_P3            : std_logic;
signal write_en_int           : std_logic;

signal clk270                 : std_logic;

signal write_data             : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data1            : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data2            : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data3            : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data4            : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data_reg_dimm    : std_logic_vector(((data_width*2)-1) downto 0);  --Added for Registered Dimms 

signal write_data270          : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data0            : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data_int         : std_logic_vector(((data_width*2)-1) downto 0);
signal write_data270_1        : std_logic_vector((data_width -1) downto 0);
signal write_data270_2        : std_logic_vector((data_width -1) downto 0);
signal write_data270_3        : std_logic_vector((data_width -1) downto 0);  --Added for Registered Dimms 

signal write_data_m0            : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_m             : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_m1            : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_m2            : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_m3            : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_m4            : std_logic_vector(((data_mask_width*2)-1) downto 0);

signal write_data_m270          : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_mask          : std_logic_vector(((data_mask_width*2)-1) downto 0);
signal write_data_mask_reg_dimm : std_logic_vector(((data_mask_width*2)-1) downto 0);


signal write_data_m270_1        : std_logic_vector((data_mask_width -1) downto 0);



signal write_data_m270_2        : std_logic_vector((data_mask_width -1) downto 0);



signal write_data_m270_3        : std_logic_vector((data_mask_width -1) downto 0);



begin

clk270 <= not clk90;

write_data0   <= user_input_data;  
write_data_m0 <= user_data_mask;

process(clk90)
begin
  if clk90'event and clk90 = '1' then 
      write_data1 <= write_data0;
	write_data2 <= write_data1;
	write_data3 <= write_data2;
	write_data4 <= write_data3;
  end if;
end process;

process(clk90)
begin
  if clk90'event and clk90 = '1' then 
	write_data_m1 <= write_data_m0;   
	write_data_m2 <= write_data_m1;     
	write_data_m3 <= write_data_m2; 
	write_data_m4 <= write_data_m3; 
  end if;
end process;


process(clk90)
begin
  if clk90'event and clk90 = '1' then 
	write_data270 <= write_data4;
  end if;
end process;



process(clk90)
begin
  if clk90'event and clk90 = '1' then 
	write_data_m270 <= write_data_m4;
  end if;
end process;


process(clk90)
begin
  if clk90'event and clk90 = '1' then 
	write_data	<= write_data270; -- pipeline varables 
	write_data_reg_dimm <= write_data;  -- Added for Registered Dimms      
  end if;
end process;

process(clk90)
begin
  if clk90'event and clk90 = '1' then 
	write_data_mask	<= write_data_m270; -- pipeline varables m
      write_data_mask_reg_dimm <= write_data_mask;   --Added for Registered Dimms  

  end if;
end process;

--process(clk270)
process(clk90)
begin
--if clk270'event and clk270 = '1' then 
if clk90'event and clk90 = '0' then 
  
	write_data270_1  <= write_data4 (((data_width*2)-1) downto data_width) ;

	write_data270_2 <= write_data270_1;  -- variable_in
 	write_data270_3 <= write_data270_2;  -- Added for Registered Dimms    

  end if;
end process;

--process(clk270)
process(clk90)
begin
--if clk270'event and clk270 = '1' then 
if clk90'event and clk90 = '0' then 

	write_data_m270_1 <= write_data_m4 (((data_mask_width*2)-1) downto data_mask_width);


     write_data_m270_2 <= write_data_m270_1;  -- variable_in
     write_data_m270_3 <= write_data_m270_2;  -- Added for Registered Dimms    

  end if;
end process;



	write_data_rising  <= write_data270_2;  




	write_data_falling <= write_data((data_width -1) downto 0);


--write_data_rising  <= write_data270_2;
--write_data_rising  <= write_data270_3;  -- For Reg Dimms
--write_data_falling <= write_data((data_width -1) downto 0);
--write_data_falling <= write_data_reg_dimm((data_width -1) downto 0); --For Reg Dimm


--data_mask_r <= write_data_m270_2;
--data_mask_r <= write_data_m270_3; -- For Reg Dimm



        data_mask_r <= write_data_m270_2;




	data_mask_f <= write_data_mask((data_mask_width -1) downto 0);




-- data path for write enable
process(clk90)
begin
  if clk90'event and clk90 = '1' then 
     write_en_P1 <= write_enable;
     write_en_P2 <= write_en_P1;
     write_en_P3 <= write_en_P2;
  end if;
end process;

-- data path for write enable

--process(clk270)
process(clk90)
begin
--if clk270'event and clk270 = '1' then 
if clk90'event and clk90 = '0' then 
    write_en_int   <= write_en_P2;
    write_en_val   <= write_en_P1;
 end if;
end process;

end arc;

