--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       ddr1_top
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The main controller state machine that controlls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - handles the data path during READ and WRITEs.
--                    - Generates control signals for other modules, including the
--                      data strobe(DQS) signal
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--library synplify; 
--use synplify.attributes.all;
--
--use work.parameter_64bit_00.all;
use work.xst_vhdl_bl4cl3_parameters_0.all;
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity   xst_vhdl_bl4cl3_top_0 is
port(
auto_ref_req          : out std_logic; 
wait_200us            : in std_logic; 
rst_dqs_div_in        : in std_logic;
rst_dqs_div_out       : out std_logic;
--reset_in              : in std_logic;   
user_input_data       : in std_logic_vector(((data_width*2)-1) downto 0);
user_data_mask        : in std_logic_vector(((data_mask_width*2) -1) downto 0);
user_output_data      : out std_logic_vector(((data_width*2) -1) downto 0):=(OTHERS => 'Z');
user_data_valid       : out std_logic;
--user_input_address    : in std_logic_vector(((row_address  + column_address )-1) downto 0);
  user_input_address    : in std_logic_vector(((row_address  + col_ap_width + bank_address )-1) downto 0);

--user_bank_address     : in std_logic_vector((bank_address -1) downto 0);
user_config_register  : in std_logic_vector(9 downto 0);
user_command_register : in std_logic_vector(2 downto 0);
user_cmd_ack          : out std_logic;      
burst_done            : in std_logic;
init_val              : out std_logic;
ar_done               : out std_logic;

     


DDR_DQS           : inout STD_LOGIC_VECTOR((data_strobe_width -1) downto 0);


DDR_DQ                : inout std_logic_vector((data_width -1) downto 0):= (OTHERS => 'Z');
DDR_CKE               : out std_logic;
DDR_CS_N               : out std_logic;
DDR_RAS_N              : out std_logic;
DDR_CAS_N              : out std_logic;
DDR_WE_N               : out std_logic;

DDR_DM            : out STD_LOGIC_VECTOR(((data_mask_width) -1) downto 0); 


DDR_BA                : out std_logic_vector((bank_address -1) downto 0);
DDR_A           : out std_logic_vector((row_address -1) downto 0);



 DDR_CK           : out STD_LOGIC;
 DDR_CK_N         : out STD_LOGIC;


clk_int	              :	in  std_logic; 
clk90_int	      :	in  std_logic; 
--clk180                : in  std_logic; -- signal added to entity
--clk270                : in  std_logic; -- signal added to entity
delay_sel_val         : in  std_logic_vector(4 downto 0); 
sys_rst_val		      :	in  std_logic; 
sys_rst90_val	      :	in  std_logic; 
sys_rst180_val	      :	in  std_logic; 
sys_rst270_val	      :	in  std_logic;
clk_out	              :	out  std_logic; 
clk90_out	      :	out  std_logic; 
sys_rst_out		      :	out std_logic; 
sys_rst90_out	      :	out  std_logic
 
);
end   xst_vhdl_bl4cl3_top_0 ;  

architecture   arc of   xst_vhdl_bl4cl3_top_0 is

component xst_vhdl_bl4cl3_controller_0
port(
auto_ref_req     : out std_logic; 
wait_200us       : in std_logic; 
clk              : in std_logic;
--clk180           : in std_logic; -- signal added
rst0             : in std_logic;
rst180           : in std_logic;
--address          : in std_logic_vector(((row_address  + column_address )-1) downto 0);
  address          : in std_logic_vector(((row_address  + col_ap_width   )-1) downto 0);

bank_address1    : in std_logic_vector((bank_address -1) downto 0);
config_register  : in std_logic_vector(9 downto 0);
command_register : in std_logic_vector(2 downto 0);
burst_done       : in std_logic;
ddr_rasb_cntrl   : out std_logic;
ddr_casb_cntrl   : out std_logic;
ddr_web_cntrl    : out std_logic;
ddr_ba_cntrl     : out std_logic_vector((bank_address -1) downto 0);
ddr_address_cntrl: out std_logic_vector((row_address -1) downto 0);
ddr_cke_cntrl    : out std_logic;
ddr_csb_cntrl    : out std_logic;
dqs_enable       : out std_logic;
dqs_reset        : out std_logic;
write_enable     : out std_logic;
rst_calib        : out std_logic;
rst_dqs_div_int  : out std_logic;
cmd_ack          : out std_logic;
init             : out std_logic;
ar_done          : out std_logic
);
end component;  

component	xst_vhdl_bl4cl3_data_path_0
port(
user_input_data    : in std_logic_vector(((data_width*2) -1) downto 0);
user_data_mask     : in std_logic_vector(((data_mask_width*2) -1) downto 0);
clk                : in std_logic;
clk90              : in std_logic;
--clk180             : in std_logic; -- signal added 
--clk270             : in std_logic; -- signal added
reset              : in std_logic;
reset90            : in std_logic;
reset180           : in std_logic;
reset270           : in std_logic;
write_enable       : in std_logic;
rst_dqs_div_in     : in std_logic;
delay_sel          : in std_logic_vector(4 downto 0);


        dqs_int_delay_in0      : in std_logic;

        dqs_int_delay_in1      : in std_logic;
 
dq                 : in std_logic_vector((data_width -1) downto 0);       
u_data_val         : out std_logic;
user_output_data   : out std_logic_vector(((data_width*2) -1) downto 0);
write_en_val       : out std_logic;
reset90_r_val      : out std_logic;

     data_mask_f       : out std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : out std_logic_vector((data_mask_width-1) downto 0);



write_data_falling : out std_logic_vector((data_width -1) downto 0);
write_data_rising  : out std_logic_vector((data_width -1) downto 0)
);
end component;

component  xst_vhdl_bl4cl3_infrastructure
port(
sys_rst            : in std_logic;
clk_int            : in std_logic;
rst_calib1         : in std_logic;
delay_sel_val      : in std_logic_vector(4 downto 0);
delay_sel_val1_val : out std_logic_vector(4 downto 0)
);
end component;

component  xst_vhdl_bl4cl3_iobs_0
port(
clk               : in std_logic;
clk90             : in std_logic;  
--clk180            : in std_logic; -- signal added 
--clk270            : in std_logic; -- signal added
ddr_rasb_cntrl    : in std_logic;
ddr_casb_cntrl    : in std_logic;
ddr_web_cntrl     : in std_logic;
ddr_cke_cntrl     : in std_logic;
ddr_csb_cntrl     : in std_logic;
ddr_address_cntrl : in std_logic_vector((row_address -1) downto 0);
ddr_ba_cntrl      : in std_logic_vector((bank_address -1) downto 0);
rst_dqs_div_int   : in std_logic;
dqs_reset         : in std_logic;
dqs_enable        : in std_logic;   

ddr_dqs               : inout std_logic_vector((data_strobe_width -1) downto 0);


ddr_dq            : inout std_logic_vector((data_width -1) downto 0);
write_data_falling: in std_logic_vector((data_width -1) downto 0);
write_data_rising : in std_logic_vector((data_width -1) downto 0);
write_en_val      : in std_logic;
reset90_r         : in std_logic;

     data_mask_f       : in std_logic_vector((data_mask_width-1) downto 0);



      data_mask_r       : in std_logic_vector((data_mask_width-1) downto 0);





 DDR_CK           : out STD_LOGIC;
 DDR_CK_N         : out STD_LOGIC;


ddr_rasb          : out std_logic;
ddr_casb          : out std_logic;
ddr_web           : out std_logic;
ddr_ba            : out std_logic_vector((bank_address -1) downto 0);
ddr_address       : out std_logic_vector((row_address -1) downto 0);
ddr_cke           : out std_logic;
ddr_csb           : out std_logic;
rst_dqs_div       : out std_logic;
rst_dqs_div_in    : in std_logic;
rst_dqs_div_out   : out std_logic;


     dqs_int_delay_in0  : out std_logic ; 

     dqs_int_delay_in1  : out std_logic ; 

ddr_dm                : out std_logic_vector((data_mask_width-1) downto 0);


dq                : out std_logic_vector((data_width -1) downto 0)
);
end component;
-------
signal rst_calib          : std_logic;
signal delay_sel          : std_logic_vector(4 downto 0);
-----
signal write_enable       : std_logic;
signal dqs_div_rst        : std_logic;
signal dqs_enable         : std_logic;
signal dqs_reset          : std_logic;


       signal dqs_int_delay_in0       : std_logic; 

       signal dqs_int_delay_in1       : std_logic; 

signal dq                 : std_logic_vector((data_width -1) downto 0);
signal write_en_val       : std_logic;
signal reset90_r          : std_logic;

signal data_mask_f        : std_logic_vector((data_mask_width -1) downto 0);



 signal data_mask_r        : std_logic_vector((data_mask_width -1) downto 0);


signal write_data_falling : std_logic_vector((data_width -1) downto 0);
signal write_data_rising  : std_logic_vector((data_width -1) downto 0);
-----
signal ddr_rasb_cntrl   :  std_logic;
signal ddr_casb_cntrl   :  std_logic;
signal ddr_web_cntrl    :  std_logic;
signal ddr_ba_cntrl     :  std_logic_vector((bank_address -1) downto 0);
signal ddr_address_cntrl:  std_logic_vector((row_address -1) downto 0);
signal ddr_cke_cntrl    :  std_logic;
signal ddr_csb_cntrl    :  std_logic;
signal rst_dqs_div_int  :  std_logic;


begin


clk_out	          <=  clk_int after 1 ps; 
clk90_out	          <=  clk90_int after 1 ps; 
sys_rst_out		    <=  sys_rst_val; 
sys_rst90_out	    <=  sys_rst90_val; 


controller0 : xst_vhdl_bl4cl3_controller_0 port map (
 auto_ref_req      => auto_ref_req, 
 wait_200us        => wait_200us, 
clk	             => clk_int, 
--clk180             => clk180,
rst0               => sys_rst_val,
rst180             => sys_rst180_val,
address            => user_input_address(((row_address  + col_ap_width + bank_address )-1) downto bank_address),
bank_address1      =>   user_input_address(bank_address - 1 downto 0),
config_register    => user_config_register,
command_register   => user_command_register,
burst_done         => burst_done,
ddr_rasb_cntrl     => ddr_rasb_cntrl,
ddr_casb_cntrl     => ddr_casb_cntrl,
ddr_web_cntrl      => ddr_web_cntrl,
ddr_ba_cntrl       => ddr_ba_cntrl,
ddr_address_cntrl  => ddr_address_cntrl,
ddr_cke_cntrl      => ddr_cke_cntrl,
ddr_csb_cntrl      => ddr_csb_cntrl,
dqs_enable         => dqs_enable,
dqs_reset          => dqs_reset,
write_enable       => write_enable,
rst_calib          => rst_calib,
rst_dqs_div_int    => rst_dqs_div_int,
cmd_ack            => user_cmd_ack,
init               => init_val,
ar_done            => ar_done
);

data_path0	:	xst_vhdl_bl4cl3_data_path_0	port	map	( 
user_input_data     =>  user_input_data,
user_data_mask      =>  user_data_mask,
clk	              =>	clk_int, 
clk90	              =>	clk90_int, 
--clk180              =>  clk180,
--clk270              =>  clk270,
reset               => sys_rst_val,
reset90             => sys_rst90_val,
reset180            => sys_rst180_val,
reset270            => sys_rst270_val,
write_enable        => write_enable,
rst_dqs_div_in      => dqs_div_rst,
delay_sel           => delay_sel,


        dqs_int_delay_in0  => dqs_int_delay_in0, 

        dqs_int_delay_in1  => dqs_int_delay_in1, 

dq                  => dq,      
u_data_val          => user_data_valid,
user_output_data    => user_output_data,
write_en_val        => write_en_val,
reset90_r_val       => reset90_r,
data_mask_f         => data_mask_f,
data_mask_r         => data_mask_r,
write_data_falling  => write_data_falling,
write_data_rising   => write_data_rising                     
);                           

infrastructure0 :  xst_vhdl_bl4cl3_infrastructure port map
(
sys_rst            =>   sys_rst_val,
clk_int            =>   clk_int,
rst_calib1         =>   rst_calib,
delay_sel_val      =>   delay_sel_val,
delay_sel_val1_val =>   delay_sel
); 

iobs0	:	xst_vhdl_bl4cl3_iobs_0	port	map 
(
clk	=>	clk_int, 
clk90	=>	clk90_int, 
--clk180             => clk180,
--clk270             => clk270,
ddr_rasb_cntrl     =>  ddr_rasb_cntrl, 
ddr_casb_cntrl     =>  ddr_casb_cntrl,
ddr_web_cntrl      =>  ddr_web_cntrl,
ddr_cke_cntrl      =>  ddr_cke_cntrl,
ddr_csb_cntrl      =>  ddr_csb_cntrl,
ddr_address_cntrl  =>  ddr_address_cntrl,
ddr_ba_cntrl       =>  ddr_ba_cntrl,
rst_dqs_div_int    =>  rst_dqs_div_int,
dqs_reset          =>  dqs_reset,
dqs_enable         =>  dqs_enable,
ddr_dqs            =>  ddr_dqs,
ddr_dq             =>  DDR_DQ,
write_data_falling =>  write_data_falling, 
write_data_rising  =>  write_data_rising,
write_en_val       =>  write_en_val,
reset90_r          =>  reset90_r,
data_mask_f        =>  data_mask_f, 
data_mask_r        =>  data_mask_r,

DDR_CK            =>    DDR_CK,
DDR_CK_N            =>    DDR_CK_N,

ddr_rasb           =>  DDR_RAS_N, 
ddr_casb           =>  DDR_CAS_N,
ddr_web            =>  DDR_WE_N,
ddr_ba             =>  DDR_BA,
ddr_address        =>  DDR_A,
ddr_cke            =>  DDR_CKE,
ddr_csb            =>  DDR_CS_N,
rst_dqs_div        =>  dqs_div_rst,
rst_dqs_div_in	 =>  rst_dqs_div_in,
rst_dqs_div_out    =>  rst_dqs_div_out,


        dqs_int_delay_in0  => dqs_int_delay_in0, 

        dqs_int_delay_in1  => dqs_int_delay_in1, 
--ddr_dm             =>  ddr_dm,

	 DDR_DM                =>   DDR_DM,

dq                 =>  dq
);							
end   arc ;  
